#include <iostream>
#include <math.h>
#include <time.h>
using namespace std;
#include "calc.h"
#include "screen.h"

//-----------------------------------------------------------------------------

extern int iScreenWidth; //never go over 1000, see array size
extern int iScreenHight; //never go over 1000, see array size
extern screen myScreen; //myScreen is defined in main

//-----------------------------------------------------------------------------



CCalculation::CCalculation()
{
	dx = dy = iMaxHit = 0;
	//in case no valid input by user make it work at all
	iIterations=1000000;
	iPattern=1;
	iColourType=1;
}



double CCalculation::randomNum( double dMin, double dMax )
{
	double dTemp;
	double dRange = ( dMax - dMin );
	double dNewRand;

	dTemp = rand();

	dNewRand = ( dTemp / ( double )RAND_MAX ) * dRange; //typecast otherwise it gives back NO fractions

	return ( dMin + dNewRand );
}



void CCalculation::initCalculate()
{
	srand( time( NULL ) ); //here, because it is counting seconds and I have to give it time to count further

	dx = randomNum( -1.0, 1.0 );
	dy = randomNum( -1.0, 1.0 );

	cout<<"calculating......."<<endl;
	cout<<endl;
	cout<<endl;
	
	for ( int i = 0; i < iIterations; i++ )
	{
		actualCalculate();
	}
	calculateHitsToColour( iScreenWidth, iScreenHight );
}



void CCalculation::actualCalculate()
{
	double dPI = 3.1415926535897932384626433832795;

	//---------------------------------------------------------------------
	//all possible pattern
	//---------------------------------------------------------------------

	//---------------------------------------------------------------------
	//those provided by Scott Drave's paper
	//---------------------------------------------------------------------
	if ( iPattern == 1 )  //Test Sierpinsky Triangle, functions from Scott Drave's paper
	{
		int iFuncNum = ( int )randomNum( 0, 3 );
		if ( iFuncNum == 0 )
		{
			dx = dx / 2.0;
			dy = dy / 2.0;
		}
		else if ( iFuncNum == 1 )
		{
			dx = ( dx + 1.0 ) / 2.0;
			dy = dy / 2.0;
		}
		else
		{
			dx = dx / 2.0;
			dy = (dy+1)/2.0;
		}
	}
	else if ( iPattern == 2 ) //"cloud", functions and coefficients by Scott Draves
	{
		int iFuncNum = ( int )randomNum( 0, 2 );
		if ( iFuncNum == 0 ) //spherical
		{
			dVx = ( 0.562482 * dx ) + ( -0.539599 * dy ) + ( -0.42992 );
			dVy = ( 0.397861 * dx ) + ( 0.501088 * dy ) + ( -0.112404 );
			dRadius = sqrt( ( dVx * dVx ) + ( dVy * dVy ) ); //see paper p.12, root of x*x+y*y
			dx = dVx / ( dRadius * dRadius );
			dy = dVy / ( dRadius * dRadius );
		}
		else
		{
			dVx = ( 0.830039 * dx ) + ( 0.16248 * dy ) + ( 0.91022 );
			dVy = ( -0.496174 * dx ) + ( 0.750468 * dy ) + ( 0.288389 );
			dRadius = sqrt( ( dVx * dVx ) + ( dVy * dVy ) ); //see paper p.12, root of x*x+y*y
			dx = dVx / ( dRadius * dRadius );
			dy = dVy / ( dRadius * dRadius );
		}
	}
	else if ( iPattern == 3 ) //"linear&spiral&swirl", functions and coefficients by Scott Draves
	{
		int iFuncNum=(int)randomNum(0,3);
		if (iFuncNum == 0) //linear
		{
			dVx = ( 0.98396 * dx ) + ( 0.359416 * dy ) + ( -0.85059 );
			dVy = ( 0.298328 * dx ) + ( -0.583541 * dy ) + ( 0.378754 );
			dx = dVx;
			dy = dVy;
		}
		else if ( iFuncNum == 1 ) //spiral
		{
			dVx = ( -0.900388 * dx ) + ( 0.397598 * dy ) + ( 0.465126 );
			dVy = ( 0.293063 * dx ) + ( 0.0225629 * dy ) + ( -0.277212 );
			dRadius = sqrt( ( dVx * dVx ) + ( dVy * dVy ) ); //see paper p.12, root of x*x+y*y
			dAngleTheta = atan( dVy / dVx ); //see paper p.12, arctan of y/x
			dx = ( cos( dAngleTheta ) + sin( dRadius ) ) / dRadius;
			dy = ( sin( dAngleTheta ) + cos( dRadius ) ) / dRadius;
		}
		else //swirl
		{
			dVx = ( -0.329863 * dx ) + ( -0.369381 * dy ) + ( 0.977861 );
			dVy = ( -0.0855261 * dx ) + ( -0.858379 * dy ) + ( 0.547595 );
			dRadius = sqrt( ( dVx * dVx ) + ( dVy * dVy ) ); //see paper p.12, root of x*x+y*y
			dAngleTheta = atan( dVy / dVx ); //see paper p.12, arctan of y/x
			dx = dRadius * ( cos( dAngleTheta + dRadius ) );
			dy = dRadius * ( sin( dAngleTheta + dRadius ) );
		}
	}

	//---------------------------------------------------------------------
	//Tina try outs
	//---------------------------------------------------------------------
	else if ( iPattern == 4 ) //Tina Test 1
	{
		int iFuncNum = ( int )randomNum( 0, 2 );
		if ( iFuncNum == 0 ) //diamond
		{
			dVx = dx; //not enough time to try out more coefficients
			dVy = dy;
			dRadius = sqrt( ( dVx * dVx ) + ( dVy * dVy ) ); //see paper p.12, root of x*x+y*y
			dAngleTheta = atan( dVy / dVx ); //see paper p.12, arctan of y/x
			dx = sin( dAngleTheta ) * cos( dRadius );
			dy = cos( dAngleTheta ) * sin( dRadius );
		}
		else //heart
		{
			dVx = dx;
			dVy = dy;
			dRadius = sqrt( ( dVx * dVx ) + ( dVy * dVy ) ); //see paper p.12, root of x*x+y*y
			dAngleTheta = atan( dVy / dVx ); //see paper p.12, arctan of y/x
			dx = dRadius * ( sin( dAngleTheta * dRadius ) );
			dy = -1.0 * ( dRadius * ( cos( dAngleTheta * dRadius ) ) );
		}
	}
	else if ( iPattern == 5 ) //Tina Test 2
	{
		int iFuncNum = ( int )randomNum( 0, 4 );
		if ( iFuncNum == 0 ) //discalone
		{
			dVx = dx;
			dVy = dy;
			dRadius = sqrt( ( dVx * dVx ) + ( dVy * dVy ) ); //see paper p.12, root of x*x+y*y
			dAngleTheta = atan( dVy / dVx ); //see paper p.12, arctan of y/x
			dx = ( dAngleTheta * ( sin( dPI * dRadius ) ) ) / dPI;
			dy = ( dAngleTheta * ( cos( dPI * dRadius ) ) ) / dPI;
		}
		else if ( iFuncNum == 1 ) //sinusoidalalone
		{
			dVx = dx;
			dVy = dy;
			dx = sin( dVx );
			dy = sin( dVy );
		}
		else  //spiral
		{
			dVx = dx;
			dVy = dy;
			dRadius = sqrt( ( dVx * dVx ) + ( dVy * dVy ) ); //see paper p.12, root of x*x+y*y
			dAngleTheta = atan( dVy / dVx ); //see paper p.12, arctan of y/x
			dx = ( cos( dAngleTheta ) + sin( dRadius ) ) / dRadius;
			dy = ( sin( dAngleTheta ) + cos( dRadius ) ) / dRadius;
		}
	}

	//-------------------------------------------------------------alone--------
	//chaos game, functions from Scott Drave's paper, p.12
	//coefficients from Scott Draves privious examples
	//---------------------------------------------------------------------
	else if ( iPattern == 6 )  //linear
	{
		dVx = ( 0.98396 * dx ) + ( 0.359416 * dy ) + ( -0.85059 );
		dVy = ( 0.298328 * dx ) + ( -0.583541 * dy ) + ( 0.378754 );
		dx = dVx;
		dy = dVy;
	}
	else if ( iPattern == 7 ) //sinusoidal
	{
		dVx = ( -0.900388 * dx ) + ( 0.397598 * dy ) + ( 0.465126 );
		dVy = ( 0.293063 * dx ) + ( 0.0225629 * dy ) + ( -0.277212 );
		dx = sin( dVx );
		dy = sin( dVy );
	}
	else if ( iPattern == 8 ) //spherical
	{
		dVx = ( -0.900388 * dx ) + ( 0.397598 * dy ) + ( 0.465126 );
		dVy = ( 0.293063 * dx ) + ( 0.0225629 * dy ) + ( -0.277212 );
		dRadius = sqrt( ( dVx * dVx ) + ( dVy * dVy ) );
		dx = dVx / ( dRadius * dRadius );
		dy = dVy / ( dRadius * dRadius );
	}
	else if ( iPattern == 9 ) //swirl
	{
		dVx = ( 0.125503 * dx ) + ( -0.00125 * dy ) + ( 0.004588 );
		dVy = ( -0.0850061 * dx ) + ( -0.136802 * dy ) + ( 0.430655 );
		dRadius = sqrt( ( dVx * dVx ) + ( dVy * dVy ) ); //see paper p.12, root of x*x+y*y
		dAngleTheta = atan( dVy / dVx ); //see paper p.12, arctan of y/x
		dx = dRadius * ( cos( dAngleTheta + dRadius ) );
		dy = dRadius * ( sin( dAngleTheta + dRadius ) );
	}
	else if ( iPattern == 10 ) //horseshoe
	{
		dVx = ( -0.329863 * dx ) + ( -0.369381 * dy ) + ( 0.977861 );
		dVy = ( -0.0855261 * dx ) + ( -0.858379 * dy ) + ( 0.547595 );
		dRadius = sqrt( ( dVx * dVx ) + ( dVy * dVy ) ); //see paper p.12, root of x*x+y*y
		dAngleTheta = atan( dVy / dVx ); //see paper p.12, arctan of y/x
		dx = dRadius * ( cos( 2.0 * dAngleTheta ) );
		dy = dRadius * ( sin( 2.0 * dAngleTheta ) );
	}
	else if ( iPattern == 11 ) //polar
	{
		dVx = ( -0.329863 * dx ) + ( -0.369381 * dy ) + ( 0.977861 );
		dVy = ( -0.0855261 * dx ) + ( -0.858379 * dy ) + ( 0.547595 );
		dRadius = sqrt( ( dVx * dVx ) + ( dVy * dVy ) ); //see paper p.12, root of x*x+y*y
		dAngleTheta = atan( dVy / dVx ); //see paper p.12, arctan of y/x
		dx = dAngleTheta / dPI;
		dy = dRadius - 1.0;
	}
	else if ( iPattern == 12 ) //handkerchief
	{
		dVx = ( -0.329863 * dx ) + ( -0.369381 * dy ) + ( 0.977861 );
		dVy = ( -0.0855261 * dx ) + ( -0.858379 * dy ) + ( 0.547595 );
		dRadius = sqrt( ( dVx * dVx ) + ( dVy * dVy ) ); //see paper p.12, root of x*x+y*y
		dAngleTheta = atan( dVy / dVx ); //see paper p.12, arctan of y/x
		dx = dRadius * ( sin( dAngleTheta + dRadius ) );
		dy = dRadius * ( cos( dAngleTheta - dRadius ) );
	}
	else if ( iPattern == 13 ) //heart
	{
		dVx = ( -0.329863 * dx ) + ( -0.369381 * dy ) + ( 0.977861 );
		dVy = ( -0.0855261 * dx ) + ( -0.858379 * dy ) + ( 0.547595 );
		dRadius = sqrt( ( dVx * dVx ) + ( dVy * dVy ) ); //see paper p.12, root of x*x+y*y
		dAngleTheta = atan( dVy / dVx ); //see paper p.12, arctan of y/x
		dx = dRadius * ( sin( dAngleTheta * dRadius ) );
		dy = -1.0 * ( dRadius * ( cos( dAngleTheta * dRadius ) ) );
	}
	else if ( iPattern == 14 ) //disc
	{
		dVx = ( -0.329863 * dx ) + ( -0.369381 * dy ) + ( 0.977861 );
		dVy = ( -0.0855261 * dx ) + ( -0.858379 * dy ) + ( 0.547595 );
		dRadius = sqrt( ( dVx * dVx ) + ( dVy * dVy ) ); //see paper p.12, root of x*x+y*y
		dAngleTheta = atan( dVy / dVx ); //see paper p.12, arctan of y/x
		dx = ( dAngleTheta * ( sin( dPI * dRadius ) ) ) / dPI;
		dy = ( dAngleTheta * ( cos( dPI * dRadius ) ) ) / dPI;
	}
	else if ( iPattern == 15 ) //spiral
	{
		dVx = ( -0.329863 * dx ) + ( -0.369381 * dy ) + ( 0.977861 );
		dVy = ( -0.0855261 * dx ) + ( -0.858379 * dy ) + ( 0.547595 );
		dRadius = sqrt( ( dVx * dVx ) + ( dVy * dVy ) ); //see paper p.12, root of x*x+y*y
		dAngleTheta = atan( dVy / dVx ); //see paper p.12, arctan of y/x
		dx = ( cos( dAngleTheta ) + sin( dRadius ) ) / dRadius;
		dy = ( sin( dAngleTheta ) + cos( dRadius ) ) / dRadius;
	}
	else if ( iPattern == 16 ) //hyperbolic
	{
		dVx = ( -0.329863 * dx ) + ( -0.369381 * dy ) + ( 0.977861 );
		dVy = ( -0.0855261 * dx ) + ( -0.858379 * dy ) + ( 0.547595 );
		dRadius = sqrt( ( dVx * dVx ) + ( dVy * dVy ) ); //see paper p.12, root of x*x+y*y
		dAngleTheta = atan( dVy / dVx ); //see paper p.12, arctan of y/x
		dx = ( sin( dAngleTheta ) ) / dRadius;
		dy = ( cos( dAngleTheta ) ) * dRadius;
	}
	else if ( iPattern == 17 ) //diamond
	{
		dVx = ( -0.329863 * dx ) + ( -0.369381 * dy ) + ( 0.977861 );
		dVy = ( -0.0855261 * dx ) + ( -0.858379 * dy ) + ( 0.54759);
		dRadius = sqrt( ( dVx * dVx ) + ( dVy * dVy ) ); //see paper p.12, root of x*x+y*y
		dAngleTheta = atan( dVy / dVx ); //see paper p.12, arctan of y/x
		dx = sin( dAngleTheta ) * cos( dRadius );
		dy = cos( dAngleTheta ) * sin( dRadius );
		}
	else if ( iPattern == 18 ) //ex
	{
		dVx = ( -0.329863 * dx ) + ( -0.369381 * dy ) + ( 0.977861 );
		dVy = ( -0.0855261 * dx ) + ( -0.858379 * dy ) + ( 0.547595 );
		dRadius = sqrt( ( dVx * dVx ) + ( dVy * dVy ) ); //see paper p.12, root of x*x+y*y
		dAngleTheta = atan( dVy / dVx ); //see paper p.12, arctan of y/x
		dx = dRadius * ( ( sin( dAngleTheta + dRadius ) ) * ( sin( dAngleTheta + dRadius ) ) * ( sin( dAngleTheta + dRadius ) ) );
		dy = dRadius * ( ( cos( dAngleTheta - dRadius ) ) * ( cos( dAngleTheta - dRadius ) ) * ( cos( dAngleTheta - dRadius ) ) );
	}

	//Julia doesnt work, there is no sqrt of a negative number
	/*else if ( iPattern == 19 ) //julia
	{
		double dOmega=0;  //see paper p.12, either 0 or PI if (dOmega==0) { dOmega=0; } else { dOmega=dPI; }
                double dTemp=(int) randomNum(0,2);
                if (dTemp==0)
                  double dOmega=0;
                else
                  double dOmega=dPI;
		dVx = ( -0.329863 * dx ) + ( -0.369381 * dy ) + ( 0.977861 );
		dVy = ( -0.0855261 * dx ) + ( -0.858379 * dy ) + ( 0.547595 );
		dRadius = sqrt( ( dVx * dVx ) + ( dVy * dVy ) ); //see paper p.12, root of x*x+y*y
		dAngleTheta = atan( dVy / dVx ); //see paper p.12, arctan of y/x
		dx = sqrt( dRadius * ( cos( ( dAngleTheta / 2.0 ) + dOmega ) ) );
		dy = sqrt( dRadius * ( sin( ( dAngleTheta / 2.0 ) + dOmega ) ) );
	}*/

        else if ( iPattern == 19 ) //in paper = fct 16, fisheye
	{
		dVx = ( -0.329863 * dx ) + ( -0.369381 * dy ) + ( 0.977861 );
		dVy = ( -0.0855261 * dx ) + ( -0.858379 * dy ) + ( 0.547595 );
		dRadius = sqrt( ( dVx * dVx ) + ( dVy * dVy ) ); //see paper p.12, root of x*x+y*y
		dAngleTheta = atan( dVy / dVx ); //see paper p.12, arctan of y/x
		dx = dx * ( 2.0 * dRadius / ( dRadius + 1.0 ) );
		dy = dy * ( 2.0 * dRadius / ( dRadius + 1.0 ) );
	}
	else if ( iPattern == 20 )//cosine
	{
		dVx = ( -0.329863 * dx ) + ( -0.369381 * dy ) + ( 0.977861 );
		dVy = ( -0.0855261 * dx ) + ( -0.858379 * dy ) + ( 0.547595 );
		dRadius = sqrt( ( dVx * dVx ) + ( dVy * dVy ) ); //see paper p.12, root of x*x+y*y
		dAngleTheta = atan( dVy / dVx ); //see paper p.12, arctan of y/x
		dx = cos( dx * dPI ) * cosh( dy );
		dy = -sin( dx * dPI ) * sinh( dy );
	}


	//the problem is infinity, not going over -1 to 1
	if ( ( dx > 1e10 || dx < -1e10 ) || ( dy > 1e10 || dy < -1e10 ) )
	{
		dx = randomNum( -1.0, 1.0 );
		dy = randomNum( -1.0, 1.0 );
	}
	else
	{
		//choose double for temporal variable for not loosing exact number
		dSx = dx;
		dSy = dy;

		// take functions x and y and convert to screen coordinates
		//convert only what lies within -1.0 - 1.0
		if ((dSx >= -1.0 && dSx<=1.0) && (dSy >= -1.0 && dSy<=1.0))
		{
			int iScreenX = 0; //initialise to zero, before using for actual calculaion
			iScreenX = convertToScreen( dSx, iScreenWidth );
			int iScreenY = 0;
			iScreenY = convertToScreen( dSy, iScreenHight );

			//count hits instead of putting it direcly onto screen
			myScreen.fScreenHits[iScreenX] [iScreenY] = myScreen.fScreenHits[iScreenX] [iScreenY] + 1;

			//count maximum hits for using value for scaled/logarithmic colour calculation
			if ( iMaxHit < myScreen.fScreenHits[iScreenX] [iScreenY] )
			{
				iMaxHit = myScreen.fScreenHits[iScreenX] [iScreenY];
			}
		}
	}
}



int CCalculation::convertToScreen( double dValue, int iScreenValue )
{
	int iCoord = ( dValue + 1.0 ) * ( ( float )iScreenValue / 2.0 ); //temporal variable to calculate screen coordinate

	//check to stay in limits, !!!800 array positions BUT 0-799
	if ( iCoord < 0 )
		iCoord = 0;
	if ( iCoord >= iScreenValue ) iCoord = iScreenValue - 1;
		return ( iCoord );
}




void CCalculation::calculateHitsToColour( int iSizeWidth, int iSizeHight )
{
	for ( int i = 0; i < iSizeWidth; i++ )
	{
		for ( int j = 0; j < iSizeHight; j++ )
		{
			if ( myScreen.fScreenHits[i] [j] != 0 ) //test transfer data from hit count to display array
			{
				if ( iColourType == 1 ) //Black & White
				{
					myScreen.fScreenArray[i] [j] [0] = 1.0;
					myScreen.fScreenArray[i] [j] [1] = 1.0;
					myScreen.fScreenArray[i] [j] [2] = 1.0;
					myScreen.fScreenArray[i] [j] [3] = 1.0;
				}

				//----------------------------------------------------------

				else if ( iColourType == 2 ) //logarithmic grayscale, result dependent on maximum hits at a point
				{
					myScreen.fScreenArray[i] [j] [0] = log10( ( double )myScreen.fScreenHits[i] [j] ) / ( log10( ( double )iMaxHit ) );
					myScreen.fScreenArray[i] [j] [1] = log10( ( double )myScreen.fScreenHits[i] [j] ) / ( log10( ( double )iMaxHit ) );
					myScreen.fScreenArray[i] [j] [2] = log10( ( double )myScreen.fScreenHits[i] [j] ) / ( log10( ( double )iMaxHit ) );
					myScreen.fScreenArray[i] [j] [3] = log10( ( double )myScreen.fScreenHits[i] [j] ) / ( log10( ( double )iMaxHit ) );
				}

				//-----------------------------------------------------------

				else if ( iColourType == 3 ) // traffic light colours
				{
					double dLog = log10( ( double )myScreen.fScreenHits[i] [j] ) / ( log10( ( double )iMaxHit ) ); //as in grayscale

					float fRed = 0.0;
					float fGreen = 0.0;
					float fBlue = 0.0;
					float fAlpha = 1.0;

					if ( dLog >= 0.0 && dLog < 0.25 )
					{
						fRed = ( dLog ) * 2.5;
						fGreen = 0.0;
						fBlue = 0.0;
						fAlpha = 1.0;
					}
					else if ( dLog >= 0.25 && dLog < 0.5 )
					{
						fRed = 1.0;
						fGreen = ( dLog - 0.25 ) * 2.5;
						fBlue = 0.0;
						fAlpha = 1.0;
					}
					else if ( dLog >= 0.5 && dLog < 0.75 )
					{
						fRed = 1.0 - ( ( dLog - 0.5 ) * 4.0 );
						fGreen = 1.0;
						fBlue = 0.0;
						fAlpha = 1.0;
          }
					else //light blue is not in traffic light but I try to emphasise the rare high hits
					{
						fRed = 0.0;
						fGreen = 1.0;
						fBlue = ( dLog - 0.75 ) * 4.0;
						fAlpha = 1.0;
						}
					myScreen.fScreenArray[i] [j] [0] = fRed;
					myScreen.fScreenArray[i] [j] [1] = fGreen;
					myScreen.fScreenArray[i] [j] [2] = fBlue;
					myScreen.fScreenArray[i] [j] [3] = fAlpha;
				}

				//-----------------------------------------------------------------//

				else if ( iColourType == 4 ) // rainbow colours
				{
					double dLog = log10( ( double )myScreen.fScreenHits[i] [j] ) / ( log10( ( double )iMaxHit ) ); //as in grayscale

					float fRed = 0.0;
					float fGreen = 0.0;
					float fBlue = 0.0;
					float fAlpha = 1.0;

					if ( dLog >= 0.0 && dLog < 0.1 )  //black to redish pink
					{
						fRed =( dLog ) * 10.0;
						fGreen = 0.0;
						fBlue = ( dLog ) * 5.0;
						fAlpha = 1.0;
					}
					else if ( dLog >= 0.1 && dLog < 0.2 ) //redish pink to red
					{
						fRed = 1.0;
						fGreen = 0.0;
						fBlue = 0.5-((dLog-0.1)*5.0);
						fAlpha = 1.0;
					}
          else if ( dLog >= 0.2 && dLog < 0.3 ) //red to orange
          {
            fRed = 1.0;
            fGreen = ( dLog -0.2) * 5.0;
            fBlue = 0.0;
            fAlpha = 1.0;
          }
          else if ( dLog >= 0.3 && dLog < 0.4 ) //orange to yellow}' at end of inp
          {
            fRed = 1.0;
            fGreen =(dLog-0.2) * 5.0;
            fBlue = 0.0;
            fAlpha = 1.0;
          }
          else if ( dLog >= 0.4 && dLog < 0.5 ) //yellow to green
          {
            fRed =1.0-((dLog-0.4)*10.0);
            fGreen = 1.0;
            fBlue = 0.0;
            fAlpha = 1.0;
          }
          else if ( dLog >= 0.5 && dLog < 0.6 ) //green to tourquoise
          {
            fRed =0.0;
            fGreen = 1.0;
            fBlue = (dLog-0.5)*10.0;
            fAlpha = 1.0;
          }
          else if ( dLog >= 0.6 && dLog < 0.7 ) //tourquoise to light blue
          {
            fRed =0.0;
            fGreen = 1.0-((dLog-0.6)*5.0);
            fBlue = 1.0;
            fAlpha = 1.0;
          }
          else if ( dLog >= 0.7 && dLog < 0.8 ) //light blue to blue
          {
            fRed =0.0;
            fGreen = 0.5-((dLog-0.7)*5.0);
            fBlue = 1.0;
            fAlpha = 1.0;
          }
          else if ( dLog >= 0.8 && dLog < 0.9 ) //blue to purple
          {
            fRed =(dLog-0.8)*5.0;
            fGreen = 0.0;
            fBlue = 1.0;
            fAlpha = 1.0;
          }
          else //purple to light purple
          {
            fRed = (dLog-0.8)*5.0;
            fGreen = 0.0;
            fBlue = 1.0;
            fAlpha = 1.0;
          }
          myScreen.fScreenArray[i] [j] [0] = fRed;
          myScreen.fScreenArray[i] [j] [1] = fGreen;
          myScreen.fScreenArray[i] [j] [2] = fBlue;
          myScreen.fScreenArray[i] [j] [3] = fAlpha;
        }

				//-----------------------------------------------------------------//

        else if ( iColourType == 5 ) // all blues
        {
          double dLog = log10( ( double )myScreen.fScreenHits[i] [j] ) / ( log10( ( double )iMaxHit ) ); //as in grayscale

          float fRed = 0.0;
          float fGreen = 0.0;
          float fBlue = 0.0;
          float fAlpha = 1.0;

          if ( dLog >= 0.0 && dLog < 0.1 )  //black to r=0.906, g=0.0, b=1.0
          {
              fRed =( dLog ) * 9.06;
              fGreen = 0.0;
              fBlue = ( dLog ) * 10.0;
              fAlpha = 1.0;
           }
          else if ( dLog >= 0.1 && dLog < 0.2 ) //... to r=0.563, g=0.0, b=1.0
          {
            fRed = 0.906-((dLog-0.1)*3.43);
            fGreen = 0.0;
            fBlue = 1.0;
            fAlpha = 1.0;
          }
          else if ( dLog >= 0.2 && dLog < 0.3 ) //... to r=0.345, g=0.0, b=1.0
          {
            fRed = 0.563-((dLog-0.2)*2.18);
            fGreen = 0.0;
            fBlue = 1.0;
            fAlpha = 1.0;
          }
          else if ( dLog >= 0.3 && dLog < 0.4 ) //... to r=0.094, g=0.0, b=1.0
          {
            fRed = 0.345-((dLog-0.3)*2.51);
            fGreen =0.0;
            fBlue = 1.0;
            fAlpha = 1.0;
          }
          else if ( dLog >= 0.4 && dLog < 0.5 ) //... to r=0.0, g=0.251, b=1.0
          {
            fRed =0.094-((dLog-0.4)*0.94);
            fGreen = (dLog-0.4)*2.51;
            fBlue = 1.0;
            fAlpha = 1.0;
          }
          else if ( dLog >= 0.5 && dLog < 0.6 ) //... to r=0.0, g=0.469, b=1.0
          {
            fRed =0.0;
            fGreen = 0.251+((dLog-0.5)*2.18);
            fBlue = 1.0;
            fAlpha = 1.0;
          }
          else if ( dLog >= 0.6 && dLog < 0.7 ) //... to r=0.0, g=0.655, b=1.0
          {
            fRed =0.0;
            fGreen = 0.469+((dLog-0.6)*1.86);
            fBlue = 1.0;
            fAlpha = 1.0;
          }
          else if ( dLog >= 0.7 && dLog < 0.8 ) //... to r=0.0, g=0.875, b=1.0
          {
            fRed =0.0;
            fGreen = 0.655+((dLog-0.7)*2.2);
            fBlue = 1.0;
            fAlpha = 1.0;
          }
          else if ( dLog >= 0.8 && dLog < 0.9 ) //... to r=0.0, g=1.0, b=0.937
          {
            fRed =0.0;
            fGreen = 0.875+((dLog-0.8)*1.25);
            fBlue =1.0-((dLog-0.8)*0.63);
            fAlpha = 1.0;
          }
          else //... to r=0.0, g=1.0, b=0.780
          {
            fRed = 0.0;
            fGreen = 1.0;
            fBlue = 0.937-((dLog-0.9)*1.57);
            fAlpha = 1.0;
          }
          myScreen.fScreenArray[i] [j] [0] = fRed;
          myScreen.fScreenArray[i] [j] [1] = fGreen;
          myScreen.fScreenArray[i] [j] [2] = fBlue;
          myScreen.fScreenArray[i] [j] [3] = fAlpha;
        }
      }
    }
  }
}


